//importing several modules from the d3.js library
import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format
} from 'd3';
  
// Providing  the Colors for Pie Chart for  different countries
const sectionColors = [
  		'yellow', 'orange', 'blue','red', 'green', 'purple', 'violet','seagreen','pink','brown'
		  ];

//  URL of my dataset which contains the country and thei literacy rate
const csvUrl = 'https://gist.githubusercontent.com/prachow/6f37e6a9f35bb32a01abe2e69df2a0f2/raw/9d9a236398aec1f1645f28c922fb96da2447b6d3/literacy.csv';

// A method to get  data from the URL
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};
//seleting SVG module and retreiving its attributes height and width
const svg = select("svg");
const width = + svg.attr('width');
const height = + svg.attr('height');

//Formatting for the tick label of an x-axis
const xAxisTickFormat = number =>
  	format('.4~g')(number);

// code for providing the  Pie Chart 
const render = (data, total) => {
  const pieData = pie().value(d=>d.literacy)(data);
  const colors = scaleOrdinal()
  	.domain(data)
  	.range(sectionColors);
  
//generator function using the arc method to set  inner radius of arc to 0 and the outer radius to 250
  const arcGenerator = d3.arc()
  .innerRadius(0)
  .outerRadius(250);
  
//providing the innerRadius ,outerRadius,padAngle,padRadius and their values to generated arcs  
	
  const segments = arc()
  	.innerRadius(0)
  	.outerRadius(180)
  	.padAngle(0)
  	.padRadius(0);
  
//creating a selection of path elements in  SVG container and binding the data to them.  
  const sections = svg.append("g")
  	.attr("transform", `translate(250,250)`)
  	.selectAll("path").data(pieData);
  
//creating the new path and providing the attributes to them in SVG container
  sections.enter()
    .append("path")
    .attr("d", segments)
  	.attr("fill", d => colors(d.data.literacy));
  
//adding the State name to each section  
 sections
    .enter()
    .append('text')
    .text(d => (d.data.State))
    .attr("transform", function(d) { return "translate(" + arcGenerator.centroid(d) + ")";  })
    .style("text-anchor", "middle")
  	.style('fill', 'black')
    .style("font-size", 15);
  

// creating the legend for the given data
  
  const legends = svg.append("g")
  	.attr("transform", "translate(500,100)")
    .selectAll(".legends").data(pieData);
  
  const legend = legends.enter().append("g").classed(".legends",true)
  	.attr("transform", (d,i)=>{
      return `translate(0,${(i+1)*30})`;
    });
  

  //adds the actual legend items to the g elements
  
  legend.append("rect").attr("width",20).attr("height",20)
  	.attr("fill", d => colors(d.data.literacy));
  legend.append("text")
  	.attr("x", 25)
  	.attr("y", 15)
  	.attr("class","legend_text")
  	.text(d => d.data.State);
  legend.append("text")
  	.attr("x", 250)
  	.attr("y", 15)
  	.attr("class","legend_value")
  	.text(d=>xAxisTickFormat((d.data.literacy/1))+"%");
}

// Main function
const run = async () => {
  let data = await fetchData()
  let total = 0
  data.forEach(d=>{
    d.literacy = +d.literacy;
    total += d.literacy
  });
	render(data,total);
};
run();